local ADDON_NAME = "SocialIndicators_ggFrames_Adapter"
local TARGET_UNIT_TAG = "reticleover"
local FRIEND_ICON_TEXTURE = "/esoui/art/campaign/campaignbrowser_friends.dds"
local IGNORED_ICON_TEXTURE = "SocialIndicators/images/target_ignored.dds"

local GetGuildMemberInfoForUnitTag = SocialIndicators.GetGuildMemberInfoForUnitTag
local ShouldTargetIgnoreIndicatorHide = SocialIndicators.ShouldTargetIgnoreIndicatorHide
local ShouldTargetFriendIndicatorHide = SocialIndicators.ShouldTargetFriendIndicatorHide
local ShouldTargetGuildIndicatorHide = SocialIndicators.ShouldTargetGuildIndicatorHide

local settings = nil
local targetFriendIcon = nil
local targetGuildLabel = nil
local targetGuildRankIcon = nil
local nextEventHandleIndex = 1

local function RegisterForEvent(event, callback)
	local eventHandleName = ADDON_NAME .. nextEventHandleIndex
	EVENT_MANAGER:RegisterForEvent(eventHandleName, event, callback)
	nextEventHandleIndex = nextEventHandleIndex + 1
	return eventHandleName
end

local function UnregisterForEvent(event, name)
	EVENT_MANAGER:UnregisterForEvent(name, event)
end

local function OnAddonLoaded(callback)
	local eventHandle = ""
	eventHandle = RegisterForEvent(EVENT_ADD_ON_LOADED, function(event, name)
		if(name ~= ADDON_NAME) then return end
		callback()
		UnregisterForEvent(event, name)
	end)
end

local function CreateTargetIndicatorControls()
	local target = GGF.UnitManager.unit['Target']

	local template = {}
	ZO_ShallowTableCopy(target.template.Class, template)
	template.OffsetX = -18
	template.Texture = FRIEND_ICON_TEXTURE
	targetFriendIcon = GGF.Window:CreateTexture("GGF_"..target.unitName.."FriendTx", target.frames.main, template);
	targetFriendIcon:SetHidden(true)

	template = {}
	ZO_ShallowTableCopy(target.template.Caption, template)
	template.OffsetY = 18
	targetGuildLabel = GGF.Window:CreateLabel("GGF_"..target.unitName.."GuildLb", target.frames.main, template);
	targetGuildLabel:SetHidden(true)

	template = {}
	ZO_ShallowTableCopy(target.template.RankTx, template)
	template.MyAnchor = BOTTOMRIGHT
	template.ParentAnchor = BOTTOMRIGHT
	template.OffsetY = 18
	targetGuildRankIcon = GGF.Window:CreateTexture("GGF_"..target.unitName.."GuildRankTx", target.frames.main, template)
	targetGuildRankIcon:SetHidden(true)
end

local function UpdateTargetIndicatorAlpha()
	local alpha = IsUnitInGroupSupportRange(TARGET_UNIT_TAG) and 1 or 0.5
	targetFriendIcon:SetAlpha(alpha)
	targetGuildRankIcon:SetAlpha(alpha)
	targetGuildLabel:SetAlpha(alpha)
end

local function UpdateTargetIndicators()
	if(DoesUnitExist(TARGET_UNIT_TAG) and IsUnitPlayer(TARGET_UNIT_TAG)) then
		if(not ShouldTargetIgnoreIndicatorHide()) then
			targetFriendIcon:SetTexture(IGNORED_ICON_TEXTURE)
			targetFriendIcon:SetHidden(false)
		elseif(not ShouldTargetFriendIndicatorHide()) then
			targetFriendIcon:SetTexture(FRIEND_ICON_TEXTURE)
			targetFriendIcon:SetHidden(false)
		else
			targetFriendIcon:SetHidden(true)
		end

		local shouldHide = ShouldTargetGuildIndicatorHide()
		if(not shouldHide) then
			local data = GetGuildMemberInfoForUnitTag(TARGET_UNIT_TAG)
			targetGuildRankIcon:SetTexture(GetFinalGuildRankTextureLarge(data.gid, data.rankIndex))
			targetGuildLabel:SetText(GetGuildName(data.gid))
		end
		targetGuildRankIcon:SetHidden(not settings.guildRankIconActive or shouldHide)
		targetGuildLabel:SetHidden(shouldHide)

		UpdateTargetIndicatorAlpha()
	else
		targetFriendIcon:SetHidden(true)
		targetGuildRankIcon:SetHidden(true)
		targetGuildLabel:SetHidden(true)
	end
end

OnAddonLoaded(function()
	settings = SocialIndicators_Settings.targetIndicators

	CreateTargetIndicatorControls()
	RegisterForEvent(EVENT_GROUP_SUPPORT_RANGE_UPDATE, UpdateTargetIndicatorAlpha)
	CALLBACK_MANAGER:RegisterCallback("SocialIndicators_TargetUnitFrameUpdated", UpdateTargetIndicators)
end)